/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#pragma once

#include <wx/confbase.h>
#include <wx/arrstr.h>
#include "wxMidi.h"
#include "exceptions.h"

class CMIDIController
{
public:
	// Singleton
	static CMIDIController* getInstance(wxConfigBase* configBase= NULL);
	static void destroyInstance();
	
	wxMidiOutDevice* GetOutDevice(); 
private:
    // Singleton
	static CMIDIController* g_instance;

	CMIDIController(wxConfigBase* configBase= NULL);
	~CMIDIController(void);

	// -1 if not found
	int SearchOutDevice (const wxString& name);

	void SelectOutputDevice(wxString& name, int& number);

	const wxString& GetCurrentDeviceName() { return m_outDevName; }
	int GetCurrentDeviceNum() { return m_outDevNumber; }
	void LetChooseOutDevice(); // { m_outDevNumber= -1; }

	wxConfigBase* m_configBase;
	wxString m_configPath;

	wxString m_outDevName;
	int m_outDevNumber;

	friend class CMIDIConfiguration;
};

class CMidiOut
{
public:
	CMidiOut ()
	{
		m_dev= CMIDIController::getInstance()->GetOutDevice();
		m_dev->Open(0);
	}
    ~CMidiOut ()
	{
		m_dev->Close();
		delete m_dev;
	}

	void Write(wxMidiShortMessage* pMsg) { CheckMidiError (m_dev->Write (pMsg)); }
	void Write(wxMidiSysExMessage* pMsg) { CheckMidiError (m_dev->Write (pMsg)); }
	const wxString GetName() { return m_dev->DeviceName(); }

private:
	void CheckMidiError (wxMidiError err)
	{ 
		if (err!= wxMIDI_NO_ERROR) {
			throw RunTimeException (_("Error trying to write MIDI message"));
		}
	}
	wxMidiOutDevice* m_dev;
};
