/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#ifndef PVOICEACTIVITY_H
#define PVOICEACTIVITY_H

//#include <osc/OscPacketListener.h>
#include "SimpleShapeDrawer.h"
#include "valuerange.h"
#include "oscinout.h"
#include "sdlcontroller.h"
#include "puredatapatch.h"


class CPVoiceActivity :	public osc::OscPacketListener
{
public:
	
	CPVoiceActivity();
	virtual ~CPVoiceActivity(void);

	void Run();
	void Finish();

	// Mic input
	void setMicInputControl(int v) { m_micInputControl.setValue (v); m_oscOut.SendSimpleMessage ("/micInput", (float) v); }
	CValueRangeI* getMicInputControl() { return &m_micInputControl; }
	float getMicInputEnvelope() { return m_micInputEnvelope; }

	// Sound output
	void setOutputControl(int v) { m_outputControl.setValue (v); m_oscOut.SendSimpleMessage ("/output", (float) v); }
	CValueRangeI* getOutputControl() { return &m_outputControl; }
	float getOutputEnvelope() { return m_outputEnvelope; }

	// Reverb
	void setReverb(int v) { m_reverb.setValue (v); m_oscOut.SendSimpleMessage ("/reverb", (float) v); }
	CValueRangeI* getReverb() { return &m_reverb; }

	// Chorus
	void setChorus(int v) { m_chorus.setValue (v); m_oscOut.SendSimpleMessage ("/chorus", (float) v); }
	CValueRangeI* getChorus() { return &m_chorus; }

	// Pitch shift
	void setPitchShift(int v) { 
		m_pitchShift.setValue (v);
		if (m_howlingReduction && abs(v)< 5)
			m_oscOut.SendSimpleMessage ("/pitchShift", (float) 5);
		else
			m_oscOut.SendSimpleMessage ("/pitchShift", (float) v);	
	}
	CValueRangeI* getPitchShift() { return &m_pitchShift; }

	// Distorsion
	void setDistorsion(int v) { m_distorsion.setValue (v); m_oscOut.SendSimpleMessage ("/distorsion", (float) v); }
	CValueRangeI* getDistorsion() { return &m_distorsion; }

	// Robot
	void setRobot (bool v) { m_robot= v; m_oscOut.SendSimpleMessage ("/robot", (float) v); }
	bool getRobot () { return m_robot; }

	// Phone
	void setPhone (bool v) { m_phone= v; m_oscOut.SendSimpleMessage ("/phone", (float) v); }
	bool getPhone () { return m_phone; }

	// Howling reduction
	void setHowlingReduction (bool v) { 
		m_howlingReduction= v; 
		setPitchShift(m_pitchShift.getValue());
		setEchoPitchShift(m_echoPitchShift.getValue());
	}		
	bool getHowlingReduction () { return m_howlingReduction; }

	// Echo delay
	void setEchoDelay(int v) { m_echoDelay.setValue (v); m_oscOut.SendSimpleMessage ("/echoDelay", (float) v); }
	CValueRangeI* getEchoDelay() { return &m_echoDelay; }

	// Echo pitch shift
	void setEchoPitchShift(int v) { 
		m_echoPitchShift.setValue (v); 
		if (m_howlingReduction && abs(v)< 5)
			m_oscOut.SendSimpleMessage ("/echoPitchShift", (float) 5);
		else 
			m_oscOut.SendSimpleMessage ("/echoPitchShift", (float) v); 
	}
	CValueRangeI* getEchoPitchShift() { return &m_echoPitchShift; }


private:

	virtual void ProcessMessage( const osc::ReceivedMessage& m, const IpEndpointName& remoteEndpoint );

	CSDLBackgroundDrawer m_sdlDrawer;
	CSimpleShapeDrawer m_simpleShapeDrawer;
	CPureDataPatch m_patch;
	COscOut m_oscOut;
	COscIn m_oscIn;
	
	CValueRangeI m_micInputControl;
	float m_micInputEnvelope;

	CValueRangeI m_outputControl;
	float m_outputEnvelope;

	CValueRangeI m_reverb;
	CValueRangeI m_chorus;
	CValueRangeI m_pitchShift;
	CValueRangeI m_distorsion;

	bool m_robot;
	bool m_phone;
	bool m_howlingReduction;

	CValueRangeI m_echoDelay;
	CValueRangeI m_echoPitchShift;
};

#endif