/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#include "sdlcontroller.h"

#include <wx/string.h>
#include "exceptions.h"

CSDLController* CSDLController::m_single= NULL;

CSDLController* CSDLController::getInstance()
{
	static CSDLController single;

	return &single;
}
	
CSDLController::CSDLController()
: m_bgDrawerRegistered(false)
{
}

CSDLController::~CSDLController()
{
	Close();
}

void CSDLController::Close()
{
	SDL_Quit();
}

// Tells controller that a new drawer wants to access screen.
// Return the SDL surface to draw (and starts SDL accordingly)
// or NULL if another background drawer is already registered.
SDL_Surface* CSDLController::RegisterBackgroundDrawer()
{
	if (m_bgDrawerRegistered) return NULL;
	
	if (SDL_Init(SDL_INIT_VIDEO)!= 0) 
		throw InitializationException (wxString::From8BitData(SDL_GetError()));
	
	SDL_Surface* screen = SDL_SetVideoMode(800, 600, 32, SDL_HWSURFACE|SDL_RESIZABLE|SDL_ANYFORMAT);	
	if (screen== NULL) {
		SDL_Quit();
		throw InitializationException (wxString::From8BitData(SDL_GetError()));
	}

	m_bgDrawerRegistered= true;

	//SDL_UpdateRect(screen, 0, 0, m_screen->w, m_screen->h);

	return screen;
}

void CSDLController::UnregisterBackgroundDrawer()
{
	if (m_bgDrawerRegistered) {
		Close();
		m_bgDrawerRegistered= false;
	}	
}

// Tells controller that background drawer has finished its job
// to draw to screen and manage events
void CSDLController::BackgroundFinishedDrawing(SDL_Surface *surface)
{
	assert (m_bgDrawerRegistered);

	// Update screen
	SDL_UpdateRect(surface, 0, 0, surface->w, surface->h);

	SDL_Event event; 
	while(SDL_PollEvent(&event))
	{  
		SDL_Surface *tmpS;

		// Loop until there are no events left on the queue 
		switch(event.type)
		{ 
		case SDL_VIDEORESIZE:  
			SDL_SetVideoMode(event.resize.w, event.resize.h, surface->format->BitsPerPixel, surface->flags);			
			//assert (tmpS== m_screen);
			break;
		// Ignore SDL window close
		//case SDL_QUIT:
		//	return false;
		}
	}
}

CSDLBackgroundDrawer::CSDLBackgroundDrawer()
: m_surface(NULL)
{
}

CSDLBackgroundDrawer::~CSDLBackgroundDrawer()
{
	Close();
}

SDL_Surface *CSDLBackgroundDrawer::GetSurface()
{
	if (m_surface== NULL)
		m_surface= CSDLController::getInstance()->RegisterBackgroundDrawer(); 
	
	return m_surface;
}

void CSDLBackgroundDrawer::Close()
{
	if (m_surface) { 
		CSDLController::getInstance()->UnregisterBackgroundDrawer(); 
		m_surface= NULL;
	}
}

void CSDLBackgroundDrawer::FinishedDrawing()
{
	assert (m_surface);
	CSDLController::getInstance()->BackgroundFinishedDrawing (m_surface);
}

////////////////////////////////////////////////////////////////////////////////////////////
	/*
	// Audio
	if (m_wantSDLAudio)
	{
		SDL_AudioSpec desiredAudio;

		desiredAudio.channels= 1;
		desiredAudio.format= AUDIO_S16SYS;
		desiredAudio.freq= 44100;
		desiredAudio.samples= 1024;
		desiredAudio.padding= 0;
		desiredAudio.callback= sndCallback;
		desiredAudio.userdata= this;

		if (SDL_OpenAudio(&desiredAudio, &m_audioSpec)!= 0)
			throw InitializationException (wxString::From8BitData(SDL_GetError()));
		
		if (m_audioSpec.format!= AUDIO_S16SYS)
			throw InitializationException (_("Can't initialize proper audio format."));
			
		SDL_PauseAudio(0);	// Start audio
	}*/


/*
void sndCallback (void *userdata, Uint8 *stream, int len)
{
	((CBaseActivity *) userdata)->audioCallback (stream, len);
}

void CBaseActivity::audioCallback(Uint8 * stream, int len)
{
}*/